#version 110
varying vec2 fuv;
varying vec2 transformed_uv;
varying mat2 rot;

uniform sampler2D tex_bg;
uniform int RV_tex_bg_YFLIP;
uniform sampler2D tex_facedepth_bg;
uniform int RV_tex_facedepth_bg_YFLIP;
uniform sampler2D tex_sucai;
uniform int RV_tex_sucai_YFLIP;
uniform sampler2D tex_erase;
uniform int RV_tex_erase_YFLIP;
uniform int blendtype;

uniform float warp_scaling_x;
uniform float warp_scaling_y;
uniform float intensity;

vec2 shadinguv()
{
   vec2 ret = fuv;
   //if(pass2_shadingmap_x_revertDa == 1) ret.x = 1.0 - ret.x;
   return ret;
}

vec4 mmu_texture2D(sampler2D tex, vec2 uv, int RV_YFLIP){
    if(RV_YFLIP > 0) uv.y = 1.0 - uv.y;
    return texture2D(tex, uv);
}
vec4 get_bg(){
    return mmu_texture2D(tex_bg, fuv, RV_tex_bg_YFLIP);
}
vec3 get_erase(){
    vec2 fuv_new = transformed_uv;
    if(fuv_new.x < 0.0 || fuv_new.x > 1.0 || fuv_new.y < 0.0 || fuv_new.y > 1.0)
        return vec3(1.0, 1.0, 1.0);
    return mmu_texture2D(tex_erase, fuv_new, RV_tex_erase_YFLIP).rgb;
}
vec2 _bg_offset(){
    float f = mmu_texture2D(tex_facedepth_bg, fuv, RV_tex_facedepth_bg_YFLIP).r;
    return vec2(f,f);
}
vec3 get_sucai(){
    vec2 bg_offset = _bg_offset();
    vec2 fuv_new = transformed_uv + rot * (bg_offset * 0.1 * vec2(warp_scaling_x, warp_scaling_y));

    if(fuv_new.x < 0.0 || fuv_new.x > 1.0 || fuv_new.y < 0.0 || fuv_new.y > 1.0)
        return vec3(0.0, 0.0, 0.0);

    fuv_new = clamp((fuv_new - 0.5) * 1.0 + 0.5, 0.0, 1.0);
    return mmu_texture2D(tex_sucai, fuv_new, RV_tex_sucai_YFLIP).rgb * intensity;
}
vec3 get_blending_pixel(vec4 target, vec4 blend, int type){
    vec3 res = vec3(0, 0, 0);
    for(int i = 0; i < 3; i++){
        if(type == 0){ // soft light
            if(blend[i] > 0.5) res[i] = 1.0 - (1.0 - target[i]) * ( 1.0 - ( blend[i] - 0.5));
            else res[i] = target[i]*(blend[i]+0.5);
        }else if(type == 1){ // hard light
            if(blend[i] > 0.5) res[i] = 1.0 - (1.0 - target[i]) * ( 1.0 - 2.0*( blend[i] - 0.5));
            else res[i] = target[i]*(blend[i]*2.0);
        }else if(type == 2){ // color dodge
            res[i] = clamp(target[i] / clamp(1.0 - blend[i], 0.001, 1.0), 0.0, 1.0);
        }else if(type == 3){ // linear dodge
            res[i] = clamp(target[i] + blend[i], 0.0, 1.0);
        }
    }
    res = target.rgb * (1.0 - blend.a) + res * blend.a;
    return res;
}
vec3 _apply_dark_corner(vec3 new_color, float mid, float sigma){
    float d = clamp(length(fuv - vec2(0.5, 0.5)) - mid, 0.0, 3.0);
    float dark_corner_v = exp(-d*d/sigma/sigma);

    //if(transformed_uv.x > 1.0 || transformed_uv.x < 0.0 || transformed_uv.y > 1.0 || transformed_uv.y < 0.0)
    //    return new_color * 0.;
    return new_color * dark_corner_v;
}
vec4 merge_color(vec4 bg, vec4 sucai){
    vec3 fg_color = get_blending_pixel(bg, sucai, blendtype);
    fg_color = clamp(fg_color, 0.0, 1.0);
    //vec3 flatten = _apply_dark_corner(fg_color, 0.5, 0.5);
    vec3 flatten = fg_color;
    return vec4(flatten, bg.a);
}
void main(){
    vec4 bg = get_bg();
    vec3 erase = get_erase();
    vec3 sucai = get_sucai() * erase;
    //gl_FragColor = vec4(sucai,1.0);
    //return;

    //vec2 lxq = _bg_offset();
    //gl_FragColor = vec4(vec3(lxq.x),1.0);
    //return;
    gl_FragColor = merge_color(bg, vec4(sucai, 1.0));
}
